@@ -139,9 +139,9 @@ class AsyncServer(server.Server):
                                 r = self._ok(packets, b64=b64)
                             else:
                                 r = packets
-                        except IOError:
+                        except EngineIOError:
                             if sid in self.sockets:  # pragma: no cover
-                                del self.sockets[sid]
+                                await self.disconnect(sid)
                             r = self._bad_request()
                         if sid in self.sockets and self.sockets[sid].closed:
                             del self.sockets[sid]
@@ -155,7 +155,14 @@ class AsyncServer(server.Server):
                         await socket.handle_post_request(environ)
                         r = self._ok()
                     except EngineIOError:
+                        if sid in self.sockets:  # pragma: no cover
+                            await self.disconnect(sid)
                         r = self._bad_request()
+                    except:  # pragma: no cover
+                        # for any other unexpected errors, we log the error
+                        # and keep going
+                        self.logger.exception('post request handler error')
+                        r = self._ok()
             else:
                 self.logger.warning('Method %s not supported', method)
                 r = self._method_not_found()
@@ -216,9 +223,10 @@ class AsyncServer(server.Server):
                           'pingInterval': int(self.ping_interval * 1000)})
         await s.send(pkt)
 
-        if await self._trigger_event('connect', sid, environ) is False:
-            self.logger.warning('Application rejected connection')
+        ret = await self._trigger_event('connect', sid, environ)
+        if ret is False:
             del self.sockets[sid]
+            self.logger.warning('Application rejected connection')
             return self._unauthorized()
 
         if transport == 'websocket':
@@ -243,6 +251,19 @@ class AsyncServer(server.Server):
                     ret = await self.handlers[event](*args)
                 except asyncio.CancelledError:  # pragma: no cover
                     pass
+                except:
+                    self.logger.exception(event + ' async handler error')
+                    if event == 'connect':
+                        # if connect handler raised error we reject the
+                        # connection
+                        return False
             else:
-                ret = self.handlers[event](*args)
+                try:
+                    return self.handlers[event](*args)
+                except:
+                    self.logger.exception(event + ' handler error')
+                    if event == 'connect':
+                        # if connect handler raised error we reject the
+                        # connection
+                        return False
         return ret
