@@ -1,6 +1,6 @@
 import asyncio
-import time
 import six
+import time
 
 from . import exceptions
 from . import packet
@@ -73,9 +73,9 @@ class AsyncSocket(socket.Socket):
             return await getattr(self, '_upgrade_' + transport)(environ)
         try:
             packets = await self.poll()
-        except exceptions.QueueEmpty as e:
+        except exceptions.QueueEmpty:
             await self.close(wait=False)
-            raise e
+            raise
         return packets
 
     async def handle_post_request(self, environ):
@@ -93,18 +93,12 @@ class AsyncSocket(socket.Socket):
         """Close the socket connection."""
         if not self.closed and not self.closing:
             self.closing = True
-            reraise_exc = None
-            try:
-                await self.server._trigger_event('disconnect', self.sid)
-            except Exception as e:
-                reraise_exc = e
+            await self.server._trigger_event('disconnect', self.sid)
             if not abort:
                 await self.send(packet.Packet(packet.CLOSE))
             self.closed = True
             if wait:
                 await self.queue.join()
-            if reraise_exc:
-                raise reraise_exc
 
     async def _upgrade_websocket(self, environ):
         """Upgrade the connection from polling to websocket."""
@@ -172,7 +166,6 @@ class AsyncSocket(socket.Socket):
         self.server.logger.info(
             '%s: Upgrade to websocket successful', self.sid)
 
-        reraise_exc = None
         while True:
             p = None
             wait_task = asyncio.ensure_future(ws.wait())
@@ -182,9 +175,14 @@ class AsyncSocket(socket.Socket):
                 # there is a bug (https://bugs.python.org/issue30508) in
                 # asyncio that causes a "Task exception never retrieved" error
                 # to appear when wait_task raises an exception before it gets
-                # cancelled. Calling wait_tas.exception() prevents the error
-                # from being issued.
-                wait_task.exception()
+                # cancelled. Calling wait_task.exception() prevents the error
+                # from being issued in Python 3.6, but causes other errors in
+                # other versions, so we run it with all errors suppressed and
+                # hope for the best.
+                try:
+                    wait_task.exception()
+                except:
+                    pass
                 break
             except:
                 break
@@ -198,15 +196,11 @@ class AsyncSocket(socket.Socket):
                 await self.receive(pkt)
             except exceptions.UnknownPacketError:
                 pass
-            except Exception as e:
-                # if we get an unexpected exception (such as something in an
-                # application event handler) we close the connection properly
-                # and then reraise the exception
-                reraise_exc = e
-                break
+            except:  # pragma: no cover
+                # if we get an unexpected exception we log the error and exit
+                # the connection properly
+                self.server.logger.exception('Receive error')
 
         await self.queue.put(None)  # unlock the writer task so it can exit
         await asyncio.wait_for(writer_task, timeout=None)
         await self.close(wait=True, abort=True)
-        if reraise_exc:
-            raise reraise_exc
